#include "eel.h"
#include "proc.h"
#include "bufed.h"

/******************************************************************************
 * ClearCase commands for Epsilon
 *
 * checkout (co):
 *   - prompts for a checkout comment (press enter for no comment).
 *     Comment is stored away and this module associates it with the buffer, and
 *     will provide it as the default comment when you check that buffer in.
 *     This is an unreserved checkout.
 *     
 * checkin (ci):
 *   - prompts for a comment, provides as default the comment that the
 *     buffer was checked out with.
 *     
 * uncheckout (unco):
 *   - unchecksout the file associated with the buffer.  Uses the -rm flag
 *     to remove the checked out version from your view.
 *     
 * list-checkouts (lsco):
 *   - lists your current checkouts.  Prompts for the VOB directory to search
 *     in.  Remembers it for future lsco commands.
 *
 * lsvtree (lsvtree):
 *   - does a text based version tree for the file indicated in the
 *     current buffer
 *     
 * xlsvtree (xlsvtree):
 *   - pops up the graphical version tree browser for the file
 *     indicated in the current buffer
 *     
 * list_activity (la):
 *   - Lists all the activities in the current VOB.
 *   
 * make_activity (ma):
 *   - Create a new activity to to associate with future
 *     checkin / checkout operations.
 *     Activity is set and provided as the default when checking in /
 *     out buffers.
 *     
 * set-activity (sa):
 *   - prompts user to enter an activity to associate with future
 *     checkin / checkout operations.
 *     Activity is set and provided as the default when checking in /
 *     out buffers.
 * 
 * cleartool-exec (cte):
 *   - execute any other cleartool command.  'cleartool' is prepended
 *     for you, so you only need to enter the command itself (e.g. lsco
 *     -me -all)
 *
 * ctbuf:
 *   - replaces current window with "cleartool-output" buffer
 *
 *   
 *  All output goes to a buffer called "cleartool-output".
 *  Results are also put up on a dialog box to give immediate response
 *  as to command success/failure,
 *  
 *  NOTE: you MUST have the environment variable MIXEDCASEDRIVES set in autoexec.bat
 *         (e.g.  MIXEDCASEDRIVES=a-z  (tells eps that a-z are
 *          mixed case, and not to force to lower case))
 *
 * $LOG$
 *  9/30/1999  - 1.0 Pete Holland (prholland@mediaone.net)
 *                                           Created.
 *  4/17/2000  - 1.1 Pete Holland            Added "cleartool-output" buffer
 *                                           management for nicer output.
 *  4/17/2000  - 1.2 Pete Holland            Added cte|cleartool-exec
 *                                           command
 *  6/27/2000  - 1.3 Erick Crowell (erickcrowell@yahoo.com)
 *											Added pop-up for List commands, Added
 *											support for clearcase
 *											'activities'.
 *  7/27/2000  - 1.4 Pete Holland			More modifications... compromises
 *******************************************************************************/

/* -------------------------------   Defines   ------------------------------- */

#define MAX_COMMENT_LEN 300      /* maximum size of a checkin/out comment */
#define MAX_COMMAND_LEN 500      /* maximum size of a command */
#define MAX_CHECKOUTS 200         /* maximum number of checkouts that we can store comments for */
#define MAX_TIME_BUF_LEN 80

/* -------------------------------   Globals   ------------------------------- */

char commandBuf[MAX_COMMAND_LEN + MAX_COMMENT_LEN];
char *co_comments[MAX_CHECKOUTS];
int co_bufnums[MAX_CHECKOUTS];
int lastfreebuf = 0;
char lastVOB[MAX_COMMENT_LEN];
int ct_bufnum;
int lsco_bufnum;
struct time_info ts;
char time_buf[MAX_TIME_BUF_LEN];
char popup_ret[MAX_COMMENT_LEN];

/* ---------------------- Internal Utility Functions ------------------------- */

char *getdayofweek(int day_of_week)
{
    if(day_of_week == 0)
        return "Sun";
    else if(day_of_week == 1)
        return "Mon";
    else if(day_of_week == 2)
        return "Tue";
    else if(day_of_week == 3)
        return "Wed";
    else if(day_of_week == 4)
        return "Thu";
    else if(day_of_week == 5)
        return "Fri";
    else
        return "Sat";
}

char *timestamp()
{
    time_and_day(&ts);
    sprintf(time_buf, "[%s %d/%d/%d %02d:%02d:%02d]", getdayofweek(ts.day_of_week), ts.month, ts.day, ts.year, ts.hour, ts.minute, ts.second);
    return time_buf;
}

void initialize_ctbuf()
{
    zap("cleartool-output");
    ct_bufnum = name_to_bufnum("cleartool-output");
    buffer_printf("cleartool-output", "Epsilon Cleartool Output Buffer: started on %s\n", timestamp());
    buffer_printf("cleartool-output", "-----------------------------------------------------------------------------------\n");
}

when_loading()
{
    initialize_ctbuf();
}

void setup_ct_buffer()
{
    int tmpbufnum;
    if(!exist("cleartool-output"))
    {
        initialize_ctbuf();
    }
    
    tmpbufnum = bufnum;
    bufnum = ct_bufnum;
	remove_line_highlight();
    point = size();
    bufnum = tmpbufnum;
    buffer_printf("cleartool-output", "\n");
    buffer_printf("cleartool-output", "%s\n", timestamp());
}

/* debug function */
command show_comment_state()
{
   int i;
   buffer_printf("cleartool_debug", "Last Free Buffer: %d\n", lastfreebuf);
   for(i = 0; i < MAX_CHECKOUTS; i++)
   {
      if(co_bufnums[i] != 0)
      {
         buffer_printf("cleartool_debug", "Index: %d, Bufnum: %d, Comment %s\n", i, co_bufnums[i], co_comments[i]);
      }
   }
}


void set_next_free_buf()
{
   int i;
   for(i = lastfreebuf; i < MAX_CHECKOUTS; i++)
   {
      if(co_bufnums[i] == 0)
      {
         lastfreebuf = i;
         break;
      }
   }
   /* if we go over, reset to 0, and start clean */
   if(i == MAX_CHECKOUTS)
   {
      lastfreebuf = 0;
      co_bufnums[lastfreebuf] = 0;
   }
}

int find_buf_num_index()
{
   int i;
   for(i = 0; i < MAX_CHECKOUTS; i++)
   {
      if(bufnum == co_bufnums[i])
      {
         return i;
      }
   }
   return -1;
}

void clear_comment(int index)
{
   if(index < 0 || index >= MAX_CHECKOUTS)
      return;
   
   co_bufnums[index] = 0;
   free(co_comments[index]);
   if(index < lastfreebuf && index != -1)
   {
      lastfreebuf = index;
   }
}

void add_comment(int index, char *comment)
{
   if(index < 0 || index >= MAX_CHECKOUTS)
      return;

   co_comments[index] = malloc(strlen(comment) + 1);
   strcpy(co_comments[index], comment);
}

command show_buf_num()
{
   buffer_printf("cleartool_debug", "bufnum: %d\n", bufnum);
}

/* ------------------- ShortCuts to Longer Command Names ------------------- */

command sa()
{
	set_activity();
}

command ma()
{
	make_activity();
}

command la()
{
	list_activity();
}

command co()
{
   checkout();
}

command lsco()
{
   list_checkouts();
}

command ci()
{
   checkin();
}

command unco()
{
   uncheckout();
}

command ctbuf()
{
    to_buffer("cleartool-output");
}

command cte()
{
	cleartool_exec();
}

/* ------------------- Cleartool Command functions ------------------- */

int get_ctbuf_size()
{
	int tmpbufnum;
	int ret_size;
	
	tmpbufnum = bufnum;
	bufnum = ct_bufnum;
	ret_size = size();
	bufnum = tmpbufnum;
	return ret_size;
}

char display_popup(char *title, int set_point, char *button_title)
{
	int tmpbufnum;
	
	popup_ret[0] = '\0';
	
	while (!user_abort) {

		if (is_gui && want_gui_prompts) {
			int win, owin = window_handle;

			save_var _doing_input = DI_VIEW;
			win = add_popup(2, 2, bufed_width, screen_lines - 6,
							ALL_BORD(BSINGLE), name_to_bufnum("cleartool-output"));

			/* go_to_link = 0; */
			make_line_highlight();
			display_dialog_box("GeneralBox", title,
							   win, 0, 0, button_title, "Cancel", "");

			point = set_point;
			view_loop(win);

			switch (key_is_button) {
				case 1:	key = 'O'; grab_full_line(name_to_bufnum("cleartool-output"), popup_ret); break;
				case 2:	key = 'C'; break;
			}
			window_handle = owin;
			remove_window(win);
			if (user_abort)
				break;
		}
		key = toupper(key);

		if (key == 'O') {
			/* okay is okay */
			break;		
		} else if (key == 'C') {
			/* Canceled */
			break;
		}
		say("");
	}
	tmpbufnum = bufnum;
	bufnum = ct_bufnum;
	remove_line_highlight();
	bufnum = tmpbufnum;
	return key;
}

command lsvtree()
{
   int cmdpoint;
   sprintf(commandBuf, "cleartool lsvtree -all %s", filename);
   setup_ct_buffer();
   cmdpoint = get_ctbuf_size();
   buffer_printf("cleartool-output", ">> %s\n", commandBuf);
   shell("", commandBuf, "cleartool-output");
   display_popup("lsvtree", cmdpoint, "OK");
}

command xlsvtree()
{
	int cmdpoint;
	sprintf(commandBuf, "cleartool lsvtree -graphical -all %s", filename);
	setup_ct_buffer();
	cmdpoint = get_ctbuf_size();
	buffer_printf("cleartool-output", ">> %s\n", commandBuf);
	shell("", commandBuf, "cleartool-output");
	display_popup(commandBuf, cmdpoint, "OK");
}

command cleartool_exec()
{
	int cmdpoint;
    char tmpCmdBuf[MAX_COMMAND_LEN];
    get_string(tmpCmdBuf, "Cleartool Command> ");
    if(tmpCmdBuf[0] == '\0')
        return;
    
    sprintf(commandBuf, "cleartool %s", tmpCmdBuf);
    setup_ct_buffer();
	cmdpoint = get_ctbuf_size();
    buffer_printf("cleartool-output", ">> %s\n", commandBuf);
    shell("", commandBuf, "cleartool-output");
	display_popup(commandBuf, cmdpoint, "OK");
}

command list_checkouts()
{
   char comment[MAX_COMMENT_LEN];
   int cmdpoint;
   if(lastVOB == NULL)
   {
      get_file_dir(comment, "VOB Directory> ");
   }
   else
   {
      get_file(comment, "VOB Directory> ", lastVOB);
   }
   chdir(comment);
   strcpy(lastVOB, comment);
   
   setup_ct_buffer();
   cmdpoint = get_ctbuf_size();
   buffer_printf("cleartool-output", ">> lsco -me -all\n");
   shell("", "cleartool lsco -fmt \"%d %u %n\\n\\tComment: %c\\n \" -me -all", "cleartool-output");
   
   display_popup("lsco -me -all", cmdpoint, "OK");
}

command list_activity()
{
	int cmdpoint;
	char comment[MAX_COMMENT_LEN];
	
	if(lastVOB == NULL)
	{
		get_file_dir(comment, "VOB Directory> ");
	}
	else
	{
		get_file(comment, "VOB Directory> ", lastVOB);
	}
	chdir(comment);
	strcpy(lastVOB, comment);
	setup_ct_buffer();
	cmdpoint = get_ctbuf_size();
	shell("", "cleartool lsact -short -cview", "cleartool-output");

	display_popup("lsact -short -cview", cmdpoint, "OK");
}

command set_activity()
{
	char comment[MAX_COMMENT_LEN];
	char activity[MAX_COMMENT_LEN];
	char ret;
	int cmdpoint;
	activity[0] = '\0';
	
	if(lastVOB == NULL)
	{
		get_file_dir(comment, "VOB Directory> ");
	}
	else
	{
		get_file(comment, "VOB Directory> ", lastVOB);
	}
	chdir(comment);
	strcpy(lastVOB, comment);
	setup_ct_buffer();
	cmdpoint = get_ctbuf_size();
	shell("", "cleartool lsact -short -cview", "cleartool-output");

	ret = display_popup("select-activity", cmdpoint, "Select Activity");

	switch(ret)
	{
		case 'O':
				strcpy(activity, popup_ret);
				break;
		case 'C':
				activity[0] = '\0';
				break;
	}
	
	if(activity[0] != '\0')
	{
		int index;

		index = find_buf_num_index();
		sprintf(commandBuf, "cleartool setactivity -nc %s", activity);
		setup_ct_buffer();
		cmdpoint = get_ctbuf_size();
		buffer_printf("cleartool-output", ">> %s\n", commandBuf);
		shell("", commandBuf, "cleartool-output");
		display_popup(commandBuf, cmdpoint, "OK");
	}
	
	
}

command make_activity()
{
	int cmdpoint;
	char comment[MAX_COMMENT_LEN];
	char activity[MAX_COMMENT_LEN];
	activity[0] = '\0';
	
	if(lastVOB == NULL)
	{
		get_file_dir(comment, "VOB Directory> ");
	}
	else
	{
		get_file(comment, "VOB Directory> ", lastVOB);
	}
	chdir(comment);
	strcpy(lastVOB, comment);
	get_string(activity, "New UCM Activity> ");
	if(activity[0] != '\0')
	{
		sprintf(commandBuf, "cleartool mkactivity -nc -headline %s %s", activity, activity);
		setup_ct_buffer();
		cmdpoint = get_ctbuf_size();
		buffer_printf("cleartool-output", ">> %s\n", commandBuf);
		shell("", commandBuf, "cleartool-output");
		display_popup(commandBuf, cmdpoint, "OK");
	}
	
}

command checkout()
{
   int cmdpoint;
   char comment[MAX_COMMENT_LEN];
   comment[0] = '\0';
   
   get_string(comment, "CO Comment> ");
   if(comment[0] == '\0')
   {
      sprintf(commandBuf, "cleartool co -unr -nc %s", filename);
   }
   else
   {
      co_bufnums[lastfreebuf] = bufnum;
      add_comment(lastfreebuf, comment);
      
      set_next_free_buf();
      
      sprintf(commandBuf, "cleartool co -unr -c \"%s\" %s", comment, filename);
   }
   setup_ct_buffer();
   cmdpoint = get_ctbuf_size();
   buffer_printf("cleartool-output", ">> %s\n", commandBuf);
   shell("", commandBuf, "cleartool-output");
   display_popup(commandBuf, cmdpoint, "OK");
}

command checkin()
{
   int index;
   int cmdpoint;
   char comment[MAX_COMMENT_LEN];
   comment[0] = '\0';
   index = find_buf_num_index();
   if(index == -1)
   {
      get_string(comment, "CI Comment> ");
   }
   else
   {
      get_strdef(comment, "CI Comment> ", co_comments[index]);
   }
   if(comment[0] == '\0')
   {
      sprintf(commandBuf, "cleartool ci -nc %s", filename);
   }
   else
   {
      sprintf(commandBuf, "cleartool ci -c \"%s\" %s", comment, filename);
   }
   setup_ct_buffer();
   cmdpoint = get_ctbuf_size();
   buffer_printf("cleartool-output", ">> %s\n", commandBuf);
   shell("", commandBuf, "cleartool-output");
   
   /* clear out comment area */
   clear_comment(index);
   display_popup(commandBuf, cmdpoint, "OK");
}

command uncheckout()
{
   int index;
   int cmdpoint;
   index = find_buf_num_index();
   sprintf(commandBuf, "cleartool unco -rm %s", filename);
   setup_ct_buffer();
   cmdpoint = get_ctbuf_size();
   buffer_printf("cleartool-output", ">> %s\n", commandBuf);
   shell("", commandBuf, "cleartool-output");
   clear_comment(index);
   display_popup(commandBuf, cmdpoint, "OK");
}